@@ -22,6 +22,16 @@
 #include <inttypes.h>
 
 #include "gstmpeg2dec.h"
+ 
+/* mpeg2dec changed a struct name after 0.3.1, here's a workaround */
+/* mpeg2dec also only defined MPEG2_RELEASE after 0.3.1
+#if MPEG2_RELEASE < MPEG2_VERSION(0,3,2)
+*/
+#ifndef MPEG2_RELEASE
+typedef picture_t mpeg2_picture_t;
+typedef gint mpeg2_state_t;
+#define STATE_BUFFER 0
+#endif
 
 /* elementfactory information */
 static GstElementDetails gst_mpeg2dec_details = {
@@ -262,7 +272,7 @@ gst_mpeg2dec_alloc_buffer (GstMpeg2dec *mpeg2dec, const mpeg2_info_t *info, gint
   GstBuffer *outbuf = NULL;
   gint size = mpeg2dec->width * mpeg2dec->height;
   guint8 *buf[3], *out;
-  const picture_t *picture;
+  const mpeg2_picture_t *picture;
 
   if (mpeg2dec->peerpool) {
     outbuf = gst_buffer_new_from_pool (mpeg2dec->peerpool, 0, 0);
@@ -389,7 +399,7 @@ update_streaminfo (GstMpeg2dec *mpeg2dec)
 static void
 gst_mpeg2dec_flush_decoder (GstMpeg2dec *mpeg2dec)
 {
-  gint state;
+  mpeg2_state_t state;
 
   if (mpeg2dec->decoder) {
     const mpeg2_info_t *info = mpeg2_info (mpeg2dec->decoder);
@@ -402,7 +412,7 @@ gst_mpeg2dec_flush_decoder (GstMpeg2dec *mpeg2dec)
 	}
       }
     } 
-    while (state != -1);
+    while (state != STATE_BUFFER && state != -1);
   }
 }
 
@@ -414,7 +424,7 @@ gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
   guint8 *data, *end;
   gint64 pts;
   const mpeg2_info_t *info;
-  gint state;
+  mpeg2_state_t state;
   gboolean done = FALSE;
 
   if (GST_IS_EVENT (buf)) {
@@ -553,9 +563,9 @@ gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
 			(info->display_fbuf ? info->display_fbuf->id : NULL));
 
 	if (info->display_fbuf && info->display_fbuf->id) {
-          const picture_t *picture;
+          const mpeg2_picture_t *picture;
 	  gboolean key_frame = FALSE;
-	  
+  
 	  outbuf = (GstBuffer *) info->display_fbuf->id;
 	  picture = info->display_picture;
 
@@ -616,6 +626,7 @@ gst_mpeg2dec_chain (GstPad *pad, GstBuffer *buf)
         break;
       }
       /* need more data */
+      case STATE_BUFFER:
       case -1:
 	done = TRUE;
 	break;
@@ -1141,3 +1152,5 @@ GstPluginDesc plugin_desc = {
   "mpeg2dec",
   plugin_init
 };
+
+
