@@ -2,6 +2,7 @@
  
 #include "config.h"
 
+#include <errno.h>
 #include <gst/gst.h>
 /* gcc -ansi -pedantic on GNU/Linux causes warnings and errors
  * unless this is defined:
@@ -99,17 +100,29 @@ _gst_ximage_new (GstXWindow *window, int width, int height)
     return NULL;
   }
  
+  if (new->ximage->bytes_per_line == 0)
+  {
+    g_warning ("gstximage.c: bytes_per_line is zero");
+    g_warning ("gstximage.c: bits_per_pixel: %d\n", new->ximage->bits_per_pixel);
+    return NULL;
+  }
+  if (new->ximage->height == 0)
+  {
+    g_warning ("gstximage.c: height is zero");
+    return NULL;
+  }
   GST_IMAGE_SIZE (new) = new->ximage->bytes_per_line * new->ximage->height;
 
-  new->SHMInfo.shmid = shmget(IPC_PRIVATE, 
+  new->SHMInfo.shmid = shmget(IPC_PRIVATE,
 		       GST_IMAGE_SIZE (new),
 		       IPC_CREAT|0777);
 
   if (new->SHMInfo.shmid < 0) {
-    perror("shmget failed:");
+    g_warning ("gstximage.c: shmget failed: %s (%d)", strerror (errno), errno);
+
     return NULL;
   }
- 
+
   GST_IMAGE_DATA (new) = new->ximage->data = new->SHMInfo.shmaddr = shmat(new->SHMInfo.shmid, 0, 0);
 
   if (new->SHMInfo.shmaddr < 0) {
