@@ -19,6 +19,8 @@
 
 #include "config.h"
 
+#include <errno.h>
+#include <sys/shm.h> /* for SHM symbols */
 #include <gst/gst.h>
 /* gcc -ansi -pedantic on GNU/Linux causes warnings and errors
  * unless this is defined:
@@ -277,7 +279,7 @@ _gst_xvimage_new (GstXvImageType  type,
 
   if (x_shm_info->shmid == -1)
   {
-    g_warning ("shmget failed!");
+    g_warning ("gstxvimage.c: shmget failed: %s (%d)", strerror (errno), errno);
 
     XFree (image->ximage);
     g_free (image->x_shm_info);
