@@ -74,12 +74,29 @@ static void		gst_xvideosink_set_property	(GObject *object, guint prop_id,
 static void		gst_xvideosink_get_property	(GObject *object, guint prop_id, 
 							 GValue *value, GParamSpec *pspec);
 
-static GstCaps* 	gst_xvideosink_get_pad_template_caps (gboolean with_xv);
+static void		gst_xvideosink_reset 		(GstXVideoSink *xvideosink);
+
+/* these functions require the lock xvideosink->lock to be held */
+static gboolean		gst_xvideosink_xwindow_new	(GstXVideoSink *xvideosink);
+static void		gst_xvideosink_xwindow_free	(GstXVideoSink *xvideosink);
+static gboolean		gst_xvideosink_image_new	(GstXVideoSink *xvideosink);
+static void		gst_xvideosink_image_free	(GstXVideoSink *xvideosink);
+
+static void	 	gst_xvideosink_get_pad_template_caps (GstXVideoSink *xvideosink);
+static void	 	gst_xvideosink_free_pad_template_caps (GstXVideoSink *xvideosink);
 
 /* default template - initiated with class struct to allow gst-register to work
    with X running */
-static GstPadTemplate *sink_template = NULL;
-static GstCaps *formats = NULL;
+GST_PAD_TEMPLATE_FACTORY (gst_xvideosink_sink_template_factory,
+  "sink",
+  GST_PAD_SINK,
+  GST_PAD_ALWAYS,
+  GST_CAPS_NEW (
+    "videosink_sink",
+    "video/raw",
+    NULL
+  )
+)
 
 static GstElementClass *parent_class = NULL;
 static guint gst_xvideosink_signals[LAST_SIGNAL] = { 0 };
@@ -121,7 +138,7 @@ gst_xvideosink_class_init (GstXVideoSinkClass *klass)
                       G_MININT, G_MAXINT, 0, G_PARAM_READABLE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FRAMES_DISPLAYED,
     g_param_spec_int ("frames_displayed", "Frames Displayed", "The number of frames displayed so far",
-                      G_MININT,G_MAXINT, 0, G_PARAM_READWRITE));
+                      0,G_MAXINT, 0, G_PARAM_READABLE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FRAME_TIME,
     g_param_spec_int ("frame_time", "Frame time", "The interval between frames",
                       G_MININT, G_MAXINT, 0, G_PARAM_READWRITE));
@@ -163,15 +180,6 @@ gst_xvideosink_class_init (GstXVideoSinkClass *klass)
   gstelement_class->change_state = gst_xvideosink_change_state;
   gstelement_class->set_clock = gst_xvideosink_set_clock;
   gstelement_class->release_locks = gst_xvideosink_release_locks;
-  
-  /* initiate default template */
-  formats = gst_xvideosink_get_pad_template_caps (TRUE);
-  sink_template = gst_pad_template_new (
-                      "sink",
-                      GST_PAD_SINK,
-                      GST_PAD_ALWAYS,
-                      formats,
-                      NULL);
 }
 
 static void
@@ -265,13 +273,16 @@ gst_xvideosink_buffer_free (GstBufferPool *pool, GstBuffer *buffer, gpointer use
 
   if (xvideosink->image) {
     gint image_type, image_size;
+    GstXWindow *image_window;
 
     image_type = GST_IMAGE_TYPE (image);
     image_size = GST_IMAGE_SIZE (image);
+    image_window = GST_IMAGE_WINDOW (image);
 	    
     /* if size and type matches, keep the buffer */
     if (image_type == GST_IMAGE_TYPE (xvideosink->image) &&
-        image_size == GST_IMAGE_SIZE (xvideosink->image)) 
+        image_size == GST_IMAGE_SIZE (xvideosink->image) &&
+        image_window == GST_IMAGE_WINDOW (xvideosink->image)) 
     { 
       keep_buffer = TRUE;
     } 
@@ -352,80 +363,66 @@ gst_xvideosink_sinkconnect (GstPad *pad, GstCaps *caps)
   xvideosink = GST_XVIDEOSINK (gst_pad_get_parent (pad));
 
   /* we are not going to act on variable caps */
-  if (!GST_CAPS_IS_FIXED (caps))
+  if (!GST_CAPS_IS_FIXED_BP (caps))
     return GST_PAD_LINK_DELAYED;
 
-  gst_caps_get (caps, 
+  for ( ; caps != NULL; caps = caps->next) {
+    gst_caps_get (caps, 
 		  "format", &xvideosink->format,
 		  "width",  &xvideosink->width,
 		  "height", &xvideosink->height,
 		  NULL);
 
-  if (gst_caps_has_fixed_property (caps, "pixel_width")) 
-    gst_caps_get_int (caps, "pixel_width", &xvideosink->pixel_width);
-  else 
-    xvideosink->pixel_width = 1;
+    if (gst_caps_has_fixed_property (caps, "pixel_width")) 
+      gst_caps_get_int (caps, "pixel_width", &xvideosink->pixel_width);
+    else 
+      xvideosink->pixel_width = 1;
 
-  if (gst_caps_has_fixed_property (caps, "pixel_height")) 
-    gst_caps_get_int (caps, "pixel_height", &xvideosink->pixel_height);
-  else 
-    xvideosink->pixel_height = 1;
+    if (gst_caps_has_fixed_property (caps, "pixel_height")) 
+      gst_caps_get_int (caps, "pixel_height", &xvideosink->pixel_height);
+    else 
+      xvideosink->pixel_height = 1;
 
-  print_format = GULONG_FROM_LE (xvideosink->format);
+    print_format = GULONG_FROM_LE (xvideosink->format);
 
-  GST_DEBUG (0, "xvideosink: setting %08x (%4.4s) %dx%d\n", 
+    GST_DEBUG (GST_CAT_PLUGIN_INFO,
+		  "xvideosink: setting %08x (%4.4s) %dx%d\n", 
 		  xvideosink->format, (gchar*)&print_format, 
 		  xvideosink->width, xvideosink->height);
 
-  g_mutex_lock (xvideosink->lock);
-
-  /* if we have any old image settings lying around, clean that up first */
-  if (xvideosink->image) {
-    _gst_image_destroy (xvideosink->image);
-    xvideosink->image = NULL;
-
-    /* clear our imagepool too */
-    gst_xvideosink_imagepool_clear (xvideosink);
-  }
-
-  /* create a new image to draw onto */
-  if (FORMAT_IS_RGB (xvideosink->format)) {
-    xvideosink->image = GST_IMAGE (_gst_ximage_new ( 
-		      xvideosink->window, 
-		      xvideosink->width, xvideosink->height));
-  } else {
-    if (xvideosink->disable_xv) {
+    g_mutex_lock (xvideosink->lock);
+    if (!gst_xvideosink_image_new (xvideosink)) {
       g_mutex_unlock (xvideosink->lock);
-      return GST_PAD_LINK_REFUSED;
+      xvideosink->format = 0;
+      continue;
     }
 
-    xvideosink->image = GST_IMAGE (_gst_xvimage_new ( 
-		      xvideosink->format,
-		      xvideosink->window, 
-		      xvideosink->width, xvideosink->height));
-  }
+    if (xvideosink->image == NULL) {
+      /* FIXME: need better error handling? */
+      gst_element_error (GST_ELEMENT (xvideosink), "image creation failed");
+      g_mutex_unlock (xvideosink->lock);
+      xvideosink->format = 0;
+      continue;
+    }
 
-  if (xvideosink->image == NULL) {
-    /* FIXME: need better error handling? */
-    gst_element_error (GST_ELEMENT (xvideosink), "image creation failed");
-    return GST_PAD_LINK_REFUSED;
-  }
+    g_mutex_unlock (xvideosink->lock);
 
-  g_mutex_unlock (xvideosink->lock);
+    do {
+      gint real_x, real_y;
 
-  {
-    gint real_x, real_y;
-
-    gst_xvideosink_get_real_size (xvideosink, &real_x, &real_y);
+      gst_xvideosink_get_real_size (xvideosink, &real_x, &real_y);
     
-    if (xvideosink->auto_size) {
-      _gst_xwindow_resize (xvideosink->window, real_x, real_y);
-    }
-    g_signal_emit (G_OBJECT (xvideosink), gst_xvideosink_signals[SIGNAL_HAVE_SIZE], 0,
-		    real_x, real_y);
+      if (xvideosink->auto_size) {
+        _gst_xwindow_resize (xvideosink->window, real_x, real_y);
+      }
+      g_signal_emit (G_OBJECT (xvideosink), gst_xvideosink_signals[SIGNAL_HAVE_SIZE], 0,
+		     real_x, real_y);
+    } while (0);
+
+    return GST_PAD_LINK_OK;
   }
 
-  return GST_PAD_LINK_OK;
+  return GST_PAD_LINK_REFUSED;
 }
 
 static GstCaps *
@@ -435,28 +432,31 @@ gst_xvideosink_getcaps (GstPad *pad, GstCaps *caps)
 
   xvideosink = GST_XVIDEOSINK (gst_pad_get_parent (pad));
 
+  if (!xvideosink->formats) {
+    gst_xvideosink_get_pad_template_caps (xvideosink);
+  }
+
   return gst_caps_ref (xvideosink->formats);
 }
 
 static void
 gst_xvideosink_init (GstXVideoSink *xvideosink)
 {
-  xvideosink->sinkpad = gst_pad_new_from_template (sink_template, "sink");
+  xvideosink->sinkpad = gst_pad_new_from_template (
+                             GST_PAD_TEMPLATE_GET (gst_xvideosink_sink_template_factory), 
+                             "sink");
   gst_element_add_pad (GST_ELEMENT (xvideosink), xvideosink->sinkpad);
   gst_pad_set_chain_function (xvideosink->sinkpad, gst_xvideosink_chain);
   gst_pad_set_link_function (xvideosink->sinkpad, gst_xvideosink_sinkconnect);
   gst_pad_set_getcaps_function (xvideosink->sinkpad, gst_xvideosink_getcaps);
   gst_pad_set_bufferpool_function (xvideosink->sinkpad, gst_xvideosink_get_bufferpool);
 
+  /* one-time initialization */
   xvideosink->window = NULL;
   xvideosink->toplevel = TRUE;
-  xvideosink->width = 100;
-  xvideosink->height = 100;
-  xvideosink->formats = formats;
-  xvideosink->lock = g_mutex_new();
-  xvideosink->disable_xv = FALSE;
-  xvideosink->auto_size = TRUE;
-  xvideosink->correction = 0;
+  xvideosink->format = 0;
+  xvideosink->formats = NULL;
+  xvideosink->lock = g_mutex_new ();
 
   xvideosink->image_pool = NULL;
   xvideosink->pool_lock = g_mutex_new ();
@@ -465,10 +465,25 @@ gst_xvideosink_init (GstXVideoSink *xvideosink)
   xvideosink->need_new_window = FALSE;
 
   xvideosink->image = NULL;
+
+  xvideosink->width = 100;
+  xvideosink->height = 100;
+  xvideosink->disable_xv = FALSE;
+  xvideosink->auto_size = TRUE;
+
+  gst_xvideosink_reset (xvideosink);
   
   GST_FLAG_SET(xvideosink, GST_ELEMENT_THREAD_SUGGESTED);
 }
 
+/* reset properties for first use or -> NULL transition */
+static void
+gst_xvideosink_reset (GstXVideoSink *xvideosink)
+{
+  xvideosink->correction = 0;
+  xvideosink->frames_displayed = 0;
+}
+
 static void
 gst_xvideosink_set_clock (GstElement *element, GstClock *clock)
 {
@@ -519,7 +534,7 @@ gst_xvideosink_chain (GstPad *pad, GstBuffer *buf)
     switch (GST_EVENT_TYPE (event)) {
       case GST_EVENT_DISCONTINUOUS:
 	offset = GST_EVENT_DISCONT_OFFSET (event, 0).value;
-	g_print ("xvideo discont %lld\n", offset);
+	g_print ("xvideo discont %" G_GINT64_FORMAT "\n", offset);
 	gst_clock_handle_discont (xvideosink->clock, (guint64) offset);
 	break;
       default:
@@ -535,7 +550,7 @@ gst_xvideosink_chain (GstPad *pad, GstBuffer *buf)
 
     xvideosink->id = gst_clock_new_single_shot_id (xvideosink->clock, time);
 
-    GST_DEBUG (0,"videosink: clock %s wait: %llu %u\n", 
+    GST_DEBUG (0,"videosink: clock %s wait: %" G_GUINT64_FORMAT " %u\n", 
 		  GST_OBJECT_NAME (xvideosink->clock), time, GST_BUFFER_SIZE (buf));
 
     ret = gst_clock_id_wait (xvideosink->id, &jitter);
@@ -608,16 +623,10 @@ gst_xvideosink_set_property (GObject *object, guint prop_id,
       break;
     case ARG_DISABLE_XV:
       xvideosink->disable_xv = g_value_get_boolean (value);
-      xvideosink->formats = gst_xvideosink_get_pad_template_caps (!xvideosink->disable_xv);
-
-      /* FIXME */
-      sink_template = gst_pad_template_new (
-		  "sink",
-                  GST_PAD_SINK,
-  		  GST_PAD_ALWAYS,
-		  xvideosink->formats, NULL);
-
-      GST_PAD_PAD_TEMPLATE (xvideosink->sinkpad) = sink_template;
+      if (xvideosink->formats) /* get rid of old (possible now invalid) list */
+        gst_xvideosink_free_pad_template_caps (xvideosink);
+      if (gst_element_get_state (GST_ELEMENT (xvideosink)) != GST_STATE_NULL)
+        gst_xvideosink_get_pad_template_caps (xvideosink);
       break;
     case ARG_TOPLEVEL:
       xvideosink->toplevel = g_value_get_boolean (value);
@@ -627,26 +636,18 @@ gst_xvideosink_set_property (GObject *object, guint prop_id,
       break;
     case ARG_NEED_NEW_WINDOW:
       if (g_value_get_boolean (value)) {
-        g_mutex_lock (xvideosink->lock);
-        /*if (xvideosink->window)
-          _gst_xwindow_destroy (xvideosink->window);*/
-        xvideosink->window = _gst_xwindow_new (xvideosink->width, 
-			                       xvideosink->height, 
-					       xvideosink->toplevel);
-        if (!xvideosink->window) {
-	  gst_element_error (GST_ELEMENT (xvideosink), "could not create X window");
-	  /* FIXME: could use better error handling here */
+        if (xvideosink->window) {
+          g_mutex_lock (xvideosink->lock);
+          gst_xvideosink_xwindow_free (xvideosink);
+          if (gst_xvideosink_xwindow_new (xvideosink)) {
+            g_signal_emit (G_OBJECT (xvideosink), 
+		           gst_xvideosink_signals[SIGNAL_HAVE_SIZE], 0,
+		           xvideosink->width, xvideosink->height);
+          }
           g_mutex_unlock (xvideosink->lock);
-	  break;
-        }
-        xvideosink->send_xid = TRUE;
-        xvideosink->need_new_window = FALSE;
-
-        g_signal_emit (G_OBJECT (xvideosink), 
-		       gst_xvideosink_signals[SIGNAL_HAVE_SIZE], 0,
-		       xvideosink->width, xvideosink->height);
-
-        g_mutex_unlock (xvideosink->lock);
+	} else {
+	  xvideosink->need_new_window = TRUE;
+	}
       }
       break;
     default:
@@ -694,17 +695,24 @@ gst_xvideosink_get_property (GObject *object, guint prop_id, GValue *value, GPar
   }
 }
 
-static GstCaps*
-gst_xvideosink_get_pad_template_caps (gboolean with_xv)
+static void
+gst_xvideosink_free_pad_template_caps (GstXVideoSink *xvideosink)
+{
+  g_assert (xvideosink->formats != NULL);
+  
+  gst_caps_unref (xvideosink->formats);
+  xvideosink->formats = NULL;
+}
+static void
+gst_xvideosink_get_pad_template_caps (GstXVideoSink *xvideosink)
 {
   GstXWindow *window;
   GstXImage *ximage;
   GstCaps *caps = NULL;
 
-  window = _gst_xwindow_new (100, 100, FALSE);
-  if (window == NULL) {
-    return NULL;
-  }
+  g_assert (xvideosink->formats == NULL);
+
+  g_return_if_fail ((window = _gst_xwindow_new (100, 100, FALSE)));
   
   ximage = _gst_ximage_new (window, 100, 100);
   if (ximage) {
@@ -725,15 +733,18 @@ gst_xvideosink_get_pad_template_caps (gboolean with_xv)
     _gst_image_destroy (GST_IMAGE (ximage));
   }
 
-  if (with_xv && _gst_xvimage_check_xvideo ()) {
-    _gst_xvimage_init();
+  if (!xvideosink->disable_xv && _gst_xvimage_check_xvideo (window->disp)) {
+    _gst_xvimage_init(window->disp);
 
     caps = gst_caps_prepend (caps, _gst_xvimage_get_capslist ());
   }
 
   _gst_xwindow_destroy (window);
 
-  return caps;
+  gst_caps_ref (caps);
+  gst_caps_sink (caps);
+
+  xvideosink->formats = caps;
 }
 
 static GstElementStateReturn
@@ -745,14 +756,10 @@ gst_xvideosink_change_state (GstElement *element)
 
   switch (GST_STATE_TRANSITION (element)) {
     case GST_STATE_NULL_TO_READY:
-      xvideosink->window = _gst_xwindow_new (xvideosink->width, 
-		                             xvideosink->height, 
-					     xvideosink->toplevel);
-      if (!xvideosink->window) {
-	gst_element_error (element, "could not create X window");
+      if (!xvideosink->formats) /* _getcaps() might already have filled this */
+        gst_xvideosink_get_pad_template_caps (xvideosink);
+      if (!gst_xvideosink_xwindow_new (xvideosink))
 	return GST_STATE_FAILURE;
-      }
-      xvideosink->send_xid = TRUE;
       break;
     case GST_STATE_READY_TO_PAUSED:
       xvideosink->correction = 0;
@@ -762,27 +769,16 @@ gst_xvideosink_change_state (GstElement *element)
     case GST_STATE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_PAUSED_TO_READY:
+      gst_xvideosink_reset (xvideosink);
       break;
     case GST_STATE_READY_TO_NULL:
-      if (xvideosink->bufferpool)
-        gst_buffer_pool_unref (xvideosink->bufferpool);
-
-      xvideosink->bufferpool = NULL;
-
-      if (xvideosink->window) {
-	if (xvideosink->image) {
-          _gst_image_destroy (xvideosink->image);
-
-	  xvideosink->image = NULL;
-	}
-	_gst_xwindow_destroy (xvideosink->window);
-	xvideosink->window = NULL;
-      }
+      gst_xvideosink_xwindow_free (xvideosink);
+      gst_xvideosink_free_pad_template_caps (xvideosink);
       break;
   }
 
-  parent_class->change_state (element);
-
+  if (parent_class->change_state)
+    return parent_class->change_state (element);
   return GST_STATE_SUCCESS;
 }
 
@@ -800,6 +796,75 @@ gst_xvideosink_release_locks (GstElement *element)
   return TRUE;
 }
 
+static gboolean
+gst_xvideosink_xwindow_new (GstXVideoSink *xvideosink)
+{
+  xvideosink->window = _gst_xwindow_new (xvideosink->width, 
+	                                 xvideosink->height, 
+					 xvideosink->toplevel);
+  if (!xvideosink->window) {
+    gst_element_error (GST_ELEMENT (xvideosink), "could not create X window");
+    return FALSE;
+  }
+  xvideosink->send_xid = TRUE;
+  xvideosink->need_new_window = FALSE;
+
+  if (!xvideosink->format) {
+    return TRUE;
+  } else {
+    return gst_xvideosink_image_new (xvideosink);
+  }
+}
+static void
+gst_xvideosink_xwindow_free (GstXVideoSink *xvideosink)
+{
+  if (xvideosink->bufferpool) {
+    gst_buffer_pool_unref (xvideosink->bufferpool);
+    xvideosink->bufferpool = NULL;
+  }
+
+  gst_xvideosink_image_free (xvideosink);
+  if (xvideosink->window) {  
+    _gst_xwindow_destroy (xvideosink->window);
+    xvideosink->window = NULL;
+  }
+}
+static gboolean
+gst_xvideosink_image_new (GstXVideoSink *xvideosink)
+{
+  if (!xvideosink->format)
+    return FALSE;
+  
+  if (xvideosink->image)
+    gst_xvideosink_image_free (xvideosink);
+
+  /* create a new image to draw onto */
+  if (FORMAT_IS_RGB (xvideosink->format)) {
+    xvideosink->image = GST_IMAGE (_gst_ximage_new ( 
+		      xvideosink->window, 
+		      xvideosink->width, xvideosink->height));
+  } else if (xvideosink->disable_xv) {
+    return FALSE;
+  } else {
+    xvideosink->image = GST_IMAGE (_gst_xvimage_new ( 
+		      xvideosink->format,
+		      xvideosink->window, 
+		      xvideosink->width, xvideosink->height));
+  }
+
+  return TRUE;
+}
+static void
+gst_xvideosink_image_free (GstXVideoSink *xvideosink)
+{
+  if (xvideosink->image) {
+    _gst_image_destroy (xvideosink->image);
+    xvideosink->image = NULL;
+    /* clear our imagepool too */
+    gst_xvideosink_imagepool_clear (xvideosink);
+  }
+}
+
 static gboolean
 plugin_init (GModule *module, GstPlugin *plugin)
 {
@@ -811,14 +876,7 @@ plugin_init (GModule *module, GstPlugin *plugin)
   g_return_val_if_fail(factory != NULL, FALSE);
 
   gst_element_factory_add_pad_template (factory, 
-          gst_pad_template_new (
-                  "sink",
-                  GST_PAD_SINK,
-                  GST_PAD_ALWAYS,
-                  gst_caps_new ("video_caps",
-                                "video/raw",
-                                NULL),
-                  NULL));
+                                        GST_PAD_TEMPLATE_GET (gst_xvideosink_sink_template_factory));
 
   gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
 
